@@ -8,19 +8,19 @@ let%html mycontent = {|
 |}
 
 
-let mytitle = Html.pcdata "A Fabulous Web Page"
+let mytitle = Html.txt "A Fabulous Web Page"
 
 let%html mypage =
   {|<html>
      <head>
        <title>|}mytitle{|</title>
      </head>
-     <body>"mycontent"</body>
+     <body>|}[mycontent]{|</body>
    </html>
   |}
 
 let () =
   let file = open_out "index.html" in
   let fmt = Format.formatter_of_out_channel file in
-  Html.pp () fmt mypage;
+  Format.fprintf fmt "%a@." (Html.pp ~indent:true ()) mypage;
   close_out file
