@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1301, USA.
 *)
 
 (** Typesafe constructors and printers for Html documents.
@@ -22,7 +22,7 @@
     @see <http://www.w3.org/TR/html5/> W3C Recommendation *)
 
 (** Concrete implementation of Html typesafe constructors.
-    See {!modtype:Html_sigs.T}.
+    See {!module-type:Html_sigs.T}.
 *)
 include Html_sigs.Make(Tyxml_xml)(Tyxml_svg).T
   with module Xml.W = Xml_wrap.NoWrap
@@ -51,3 +51,13 @@ module Make_printer(O : Xml_sigs.Output) :
                           and type 'a elt := 'a elt
                           and type doc := doc
      [@@ocaml.deprecated "Use Html.pp instead."]
+
+(**/*)
+
+(** Toplevel printers *)
+
+val _pp : Format.formatter -> doc -> unit
+[@@ocaml.toplevel_printer]
+
+val _pp_elt : Format.formatter -> _ elt -> unit
+[@@ocaml.toplevel_printer]
