@@ -16,13 +16,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1301, USA.
 *)
 
 (** Type instantiations for SVG *)
 
 (** This module defines basic data types for data, attributes
-    and element occuring in SVG documents.
+    and element occurring in SVG documents.
     It is based on the specification available at http://www.w3.org/TR/SVG/.
 
     This module is experimental, it may lack of some attributes,
@@ -244,6 +244,8 @@ struct
 
   let a_id = string_attrib "id"
 
+  let a_user_data name = string_attrib ("data-" ^ name)
+
   let a_xml_base = string_attrib "xml:base"
 
   let a_xml_lang = string_attrib "xml:lang"
@@ -691,7 +693,7 @@ struct
   (** Javascript mouse events *)
 
   let a_onclick = Xml.mouse_event_handler_attrib "onclick"
-  let a_onmousedown = Xml.mouse_event_handler_attrib "onmousdown"
+  let a_onmousedown = Xml.mouse_event_handler_attrib "onmousedown"
   let a_onmouseup = Xml.mouse_event_handler_attrib "onmouseup"
   let a_onmouseover = Xml.mouse_event_handler_attrib "onmouseover"
   let a_onmouseout = Xml.mouse_event_handler_attrib "onmouseout"
