@@ -25,8 +25,8 @@ module String = struct
   let capitalize_ascii = String.capitalize [@ocaml.warning "-3"]
 end
 
-open Asttypes
-open Parsetree
+open Ppxlib.Asttypes
+open Ppxlib.Parsetree
 
 type lang = Common.lang = Html | Svg
 let lang_of_ns loc ns =
@@ -37,30 +37,13 @@ let lang_of_ns loc ns =
 
 module Loc = struct
 
-  let shift (pos:Lexing.position) x = {pos with pos_cnum = pos.pos_cnum + x}
-
-  let shrink {Location. loc_start ; loc_end ; loc_ghost } ~xbegin ~xend =
-    { Location.loc_ghost ;
-      loc_start = shift loc_start xbegin ;
-      loc_end = shift loc_end xend ;
-    }
-
-  (** Returns the real (OCaml) location of the content of a string, taking
-      delimiters into account. *)
-  let string_start delimiter loc =
-    let delimiter_length = match delimiter with
-      | None -> 1
-      | Some d -> String.length d + 2
-    in
-    shift loc.Location.loc_start delimiter_length
-
   (** 0-width locations do not show in the toplevel. We expand them to
       one-width.
   *)
   let one_width ?(ghost=false) pos =
     { Location.loc_ghost = ghost ;
       loc_start = pos ;
-      loc_end = shift pos 1
+      loc_end = {pos with pos_cnum = pos.pos_cnum + 1}
     }
 
   (** Given a list of input strings for Markup.ml, evaluates to a function that
@@ -251,10 +234,10 @@ let ast_to_stream expressions =
 
   let strings =
     expressions |> List.map @@ fun expr ->
-    match Ast_convenience.get_str_with_quotation_delimiter expr with
-    | Some (s, delimiter) ->
-      (s, Loc.string_start delimiter expr.pexp_loc)
-    | None ->
+    match expr.pexp_desc with
+    | Pexp_constant (Pconst_string (s, loc, _)) ->
+      (s, loc.loc_start)
+    | _ ->
       (Antiquot.create expr, expr.pexp_loc.loc_start)
   in
 
@@ -366,48 +349,28 @@ let markup_to_expr_with_implementation lang modname loc expr =
   | _ ->
     markup_to_expr lang loc expr
 
-
 let is_capitalized s =
   if String.length s < 0 then false
   else match s.[0] with
     | 'A'..'Z' -> true
     | _ -> false
 
-(** Extract and verify the modname in the annotation [%html.Bar.Baz .. ].
-    We need to fiddle with length to provide a correct location. *)
-let get_modname ~loc len l =
-  let s = String.concat "." l in
-  let loc = Loc.shrink loc ~xbegin:(len - String.length s) ~xend:0 in
-  if l = [] then None
-  else if not (List.for_all is_capitalized l) then
-    Common.error loc "This identifier is not a module name"
-  else Some s
-
-let re_dot = Re.(compile @@ char '.')
-let dispatch_ext {txt ; loc} =
-  let l = Re.split re_dot txt in
-  let len = String.length txt in
-  match l with
-  | "html" :: l
-  | "tyxml" :: "html" :: l ->
-    Some (Common.Html, get_modname ~loc len l)
-  | "svg" :: l
-  | "tyxml" :: "svg" :: l ->
-    Some (Common.Svg, get_modname ~loc len l)
-  | _ -> None
+(** Extract and verify the modname in the annotation [%html.Bar.Baz .. ]. *)
+let get_modname = function
+  | None -> None
+  | Some {txt = longident ; loc} ->
+    let l = Longident.flatten_exn longident in
+    let s = String.concat "." l in
+    if l = [] then None
+    else if not (List.for_all is_capitalized l) then
+      Common.error loc "This identifier is not a module name"
+    else Some s
 
 let application_to_list expr =
   match expr.pexp_desc with
   | Pexp_apply (f, arguments) -> f::(List.map snd arguments)
   | _ -> [expr]
 
-
-open Ast_mapper
-open Ast_helper
-
-let error { txt ; loc } =
-  Common.error loc "Invalid payload for [%%%s]" txt
-
 let markup_cases ~lang ~modname cases =
   let f ({pc_rhs} as case) =
     let loc = pc_rhs.pexp_loc in
@@ -438,37 +401,18 @@ let markup_bindings ~lang ~modname l =
   in
   List.map f l
 
-let rec expr mapper e =
+let expand_expr ~lang ~loc:_ ~path:_ ~arg e _ =
+  let modname = get_modname arg in
   match e.pexp_desc with
-  | Pexp_extension (ext, payload) ->
-    begin match dispatch_ext ext, payload with
-    | Some (lang, modname), PStr [{pstr_desc = Pstr_eval (e, _)}] ->
-      begin match e.pexp_desc with
-        | Pexp_let (recflag, bindings, next) ->
-          let bindings = markup_bindings ~lang ~modname bindings in
-          {e with pexp_desc = Pexp_let (recflag, bindings, expr mapper next)}
-        | _ ->
-          markup_to_expr_with_implementation lang modname e.pexp_loc  @@
-          application_to_list e
-      end
-    | Some _, _ -> error ext
-    | None, _ -> default_mapper.expr mapper e
-    end
-  | _ -> default_mapper.expr mapper e
-
-let structure_item mapper stri =
-  match stri.pstr_desc with
-  | Pstr_extension ((ext, payload), _attrs) ->
-    begin match dispatch_ext ext, payload with
-    | Some (lang, modname),
-      PStr [{pstr_desc = Pstr_value (recflag, bindings) }] ->
-      let bindings = markup_bindings ~lang ~modname bindings in
-      Str.value recflag bindings
-
-    | Some _, _ -> error ext
-    | None, _ -> default_mapper.structure_item mapper stri
-    end
-  | _ -> default_mapper.structure_item mapper stri
-
-let mapper _ _ =
-  {default_mapper with expr ; structure_item}
+  | Pexp_let (recflag, bindings, next) ->
+    let bindings = markup_bindings ~lang ~modname bindings in
+    {e with pexp_desc = Pexp_let (recflag, bindings, next)}
+  | _ ->
+    markup_to_expr_with_implementation lang modname e.pexp_loc @@
+    application_to_list e
+  
+let expand_str_item ~lang ~loc:_ ~path:_ ~arg recflag value_bindings =
+  let bindings =
+    markup_bindings ~lang ~modname:(get_modname arg) value_bindings
+  in 
+  Ppxlib.Ast_helper.Str.value recflag bindings
